#!/usr/local/bin/ruby -Ku

require 'rubygems'
require 'nokogiri'

module Atok_plugin
  EXCLUDE_METHODS = ['clear_generated_description',
                     'generated_description',
                     'last_matcher',
                     'last_matcher=',
                     'last_should',
                     'last_should=']

  def run_process(request)
    return nil unless ['rsm', 'ｒｓｍ'].include?(request['composition_string'])

    filenames = []  # HTML filenames to extract methods.

    # rspec
    spec_gs = Gem.searcher.find('spec')
    if spec_gs.has_rdoc?
      filenames << "#{spec_gs.installation_path}/doc/#{spec_gs.full_name}/rdoc/classes/Spec/Matchers.html"
    end

    # rspec-rails
    spec_rails_gs = Gem.searcher.find('spec/rails')
    if spec_rails_gs.has_rdoc?
      filenames << "#{spec_rails_gs.installation_path}/doc/#{spec_rails_gs.full_name}/rdoc/classes/Spec/Rails/Matchers.html"
    end

    methods = filenames.inject({}){|methods, filename| methods.merge!(extract_methods(filename))}

    candidates = methods.to_a.sort{|a, b| a[0] <=> b[0]}.map do |name, description|
      candidate = {'hyoki' => name,
                   'comment_xhtml' => '',
                   'alternative' => name,
                   'alternative_type' => 'definite_string'}

      candidate['comment_xhtml'] = <<-EOT
        <?xml version="1.0" encoding="UTF-8" ?>
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="ja" lang="ja">
          <head><title>AtokDirect</title></head>
          <body>#{description}</body>
        </html>
      EOT

      candidate
    end

    {'candidate' => candidates}
  rescue => evar
    {'candidate' => [{'hyoki' => '失敗＞＜',
                      'comment' => 'rspec と rspec-rails 、そして nokogiri の gem はインストールしましたか？ rdoc はあります？',
                      'alternative' => 'なにかエラーのようです。',
                      'alternative_type' => 'definite_string'}]}
  end

  private
  def extract_methods(filename)
    File.open(filename) do |file|
      doc = Nokogiri::HTML(file.read)
      doc.css('#method-list .name-list a').inject({}) do |methods, a_tag|
        method_id = a_tag.attributes['href'].value.sub(/^#/, '')
        method_name = a_tag.content

        next methods if EXCLUDE_METHODS.include?(method_name)

        methods.merge!({method_name => doc.css("#method-#{method_id} .method-heading .method-name").to_xhtml +
                                       doc.css("#method-#{method_id} .method-description").to_xhtml})
      end
    end
  end
end
